@@ -1,8 +1,8 @@
-/* 
-Query and display NVIDIA VDPAU capabilities, a la glxinfo 
-version 0.0.2
+/*
+Query and display NVIDIA VDPAU capabilities, a la glxinfo
 
 Copyright (c) 2008 Wladimir J. van der Laan
+Copyright (c) 2014 NVIDIA Corporation
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
@@ -25,14 +25,11 @@ WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 */
-/// TODO
-/// - possible bug, querying VDP_CHROMA_TYPE_444 returns an error
-///  (VDP_STATUS_INVALID_CHROMA_TYPE) is this NVIDIA or us?
-/// - parse display/screen from command line
-/// - list color table formats for queryOutputSurface
 
+#include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <vdpau/vdpau.h>
 #include <vdpau/vdpau_x11.h>
 
@@ -48,7 +45,7 @@ void queryBaseInfo(VDPDeviceImpl *device)
        device->GetInformationString(&info) != VDP_STATUS_OK)
     {
         fprintf(stderr, "Error querying API version or information\n");
-        exit(-1);    
+        exit(-1);
     }
     printf("API version: %i\n", api);
     printf("Information string: %s\n", info);
@@ -67,6 +64,9 @@ Desc chroma_types[] = {
 {"420", VDP_CHROMA_TYPE_420},
 {"422", VDP_CHROMA_TYPE_422},
 {"444", VDP_CHROMA_TYPE_444},
+{"420_16", VDP_CHROMA_TYPE_420_16},
+{"422_16", VDP_CHROMA_TYPE_422_16},
+{"444_16", VDP_CHROMA_TYPE_444_16},
 };
 const size_t chroma_type_count = sizeof(chroma_types)/sizeof(Desc);
 
@@ -77,6 +77,11 @@ Desc ycbcr_types[] = {
 {"YUYV", VDP_YCBCR_FORMAT_YUYV},
 {"Y8U8V8A8", VDP_YCBCR_FORMAT_Y8U8V8A8},
 {"V8U8Y8A8", VDP_YCBCR_FORMAT_V8U8Y8A8},
+{"NV24", VDP_YCBCR_FORMAT_Y_UV_444},
+{"YV24", VDP_YCBCR_FORMAT_Y_U_V_444},
+{"P010", VDP_YCBCR_FORMAT_P010},
+{"P016", VDP_YCBCR_FORMAT_P016},
+{"Y_U_V_444_16", VDP_YCBCR_FORMAT_Y_U_V_444_16},
 };
 const size_t ycbcr_type_count = sizeof(ycbcr_types)/sizeof(Desc);
 
@@ -97,11 +102,6 @@ Desc indexed_types[] = {
 };
 const size_t indexed_type_count = sizeof(indexed_types)/sizeof(Desc);
 
-Desc color_table_formats[] = {
-{"B8G8R8X8", VDP_COLOR_TABLE_FORMAT_B8G8R8X8},
-};
-const size_t color_table_format_count = sizeof(color_table_formats)/sizeof(Desc);
-
 
 void queryVideoSurface(VDPDeviceImpl *device)
 {
@@ -111,19 +111,18 @@ void queryVideoSurface(VDPDeviceImpl *device)
     printf("-------------------------------------------\n");
     for(int x=0; x<chroma_type_count; ++x)
     {
-        VdpBool is_supported = false;
+        VdpBool is_supported;
         uint32_t max_width, max_height;
 
-        rv = device->VideoSurfaceQueryCapabilities(device->device, chroma_types[x].id, 
+        rv = device->VideoSurfaceQueryCapabilities(device->device, chroma_types[x].id,
             &is_supported, &max_width, &max_height);
         if(rv == VDP_STATUS_OK && is_supported)
         {
-            printf("%-6s %5i %5i  ", chroma_types[x].name, 
+            printf("%-6s %5i %5i  ", chroma_types[x].name,
                 max_width, max_height);
             /* Find out supported formats */
             for(int y=0; y<ycbcr_type_count; ++y)
             {
-                is_supported = false;
                 rv = device->VideoSurfaceQueryGetPutBitsYCbCrCapabilities(
                     device->device, chroma_types[x].id, ycbcr_types[y].id,
                     &is_supported);
@@ -146,34 +145,46 @@ void queryOutputSurface(VDPDeviceImpl *device)
     printf("----------------------------------------------------\n");
     for(int x=0; x<rgb_type_count; ++x)
     {
-        VdpBool is_supported, native=false;
+        VdpBool is_supported, native;
         uint32_t max_width, max_height;
 
-        rv = device->OutputSurfaceQueryCapabilities(device->device, rgb_types[x].id, 
+        rv = device->OutputSurfaceQueryCapabilities(device->device, rgb_types[x].id,
             &is_supported, &max_width, &max_height);
         device->OutputSurfaceQueryGetPutBitsNativeCapabilities(device->device, rgb_types[x].id,
             &native);
         if(rv == VDP_STATUS_OK && is_supported)
         {
-            printf("%-16s %5i %5i    %c  ", rgb_types[x].name, 
+            printf("%-16s %5i %5i    %c  ", rgb_types[x].name,
                 max_width, max_height, native?'y':'-');
             /* Find out supported formats */
             for(int y=0; y<ycbcr_type_count; ++y)
             {
-                is_supported = false;
                 rv = device->OutputSurfaceQueryPutBitsYCbCrCapabilities(
-                    device->device, chroma_types[x].id, ycbcr_types[y].id,
+                    device->device, rgb_types[x].id, ycbcr_types[y].id,
                     &is_supported);
                 if(rv == VDP_STATUS_OK && is_supported)
                 {
                     printf("%s ", ycbcr_types[y].name);
                 }
             }
+
+            for(int y=0; y<indexed_type_count; ++y)
+            {
+                // There is currently only one color table format.  This will
+                // have to be reconsidered if additional color table formats are
+                // added.
+                rv = device->OutputSurfaceQueryPutBitsIndexedCapabilities(
+                    device->device, rgb_types[x].id, indexed_types[y].id,
+                    VDP_COLOR_TABLE_FORMAT_B8G8R8X8, &is_supported);
+                if(rv == VDP_STATUS_OK && is_supported)
+                {
+                    printf("%s ", indexed_types[y].name);
+                }
+            }
+
             printf("\n");
         }
     }
-    // OutputSurfaceQueryPutBitsIndexedCapabilities
-    //   rgba, idx, colortable -> supported
 }
 
 /***************** Bitmap surface ****************/
@@ -188,11 +199,11 @@ void queryBitmapSurface(VDPDeviceImpl *device)
         VdpBool is_supported;
         uint32_t max_width, max_height;
 
-        rv = device->BitmapSurfaceQueryCapabilities(device->device, rgb_types[x].id, 
+        rv = device->BitmapSurfaceQueryCapabilities(device->device, rgb_types[x].id,
             &is_supported, &max_width, &max_height);
         if(rv == VDP_STATUS_OK && is_supported)
         {
-            printf("%-16s %5i %5i\n", rgb_types[x].name, 
+            printf("%-16s %5i %5i\n", rgb_types[x].name,
                 max_width, max_height);
         }
     }
@@ -266,13 +277,14 @@ void queryVideoMixer(VDPDeviceImpl *device)
     printf("------------------------------------\n");
     for(int x=0; x<mixer_features_count; ++x)
     {
-        VdpBool is_supported = true; /* There seems to be a bug in VideoMixerQueryFeatureSupport, is_supported is only set if the feature is not supported */
-        rv = device->VideoMixerQueryFeatureSupport(device->device, mixer_features[x].id, 
+        VdpBool is_supported;
+
+        rv = device->VideoMixerQueryFeatureSupport(device->device, mixer_features[x].id,
             &is_supported);
         is_supported = (rv == VDP_STATUS_OK && is_supported);
-        printf("%-32s %c\n", mixer_features[x].name, 
+        printf("%-32s %c\n", mixer_features[x].name,
             is_supported?'y':'-');
-       
+
     }
     printf("\n");
     // Parameters (+range)
@@ -280,17 +292,18 @@ void queryVideoMixer(VDPDeviceImpl *device)
     printf("-----------------------------------------------------\n");
     for(int x=0; x<mixer_parameters_count; ++x)
     {
-        VdpBool is_supported = false;
+        VdpBool is_supported;
 
-        rv = device->VideoMixerQueryParameterSupport(device->device, mixer_parameters[x].id, 
+        rv = device->VideoMixerQueryParameterSupport(device->device, mixer_parameters[x].id,
             &is_supported);
         is_supported = (rv == VDP_STATUS_OK && is_supported);
-        printf("%-32s %c  ", mixer_parameters[x].name, 
+        printf("%-32s %c  ", mixer_parameters[x].name,
             is_supported?'y':'-');
-        if(is_supported)
+        /* VDPAU spec does not allow range query for DT_NONE types */
+        if(is_supported && mixer_parameters[x].aux != DT_NONE)
         {
             uint32_t minval, maxval;
-            rv = device->VideoMixerQueryParameterValueRange(device->device, mixer_parameters[x].id, 
+            rv = device->VideoMixerQueryParameterValueRange(device->device, mixer_parameters[x].id,
                 (void*)&minval, (void*)&maxval);
             if(rv == VDP_STATUS_OK)
                 display_range(mixer_parameters[x].aux, minval, maxval);
@@ -298,30 +311,31 @@ void queryVideoMixer(VDPDeviceImpl *device)
         printf("\n");
     }
     printf("\n");
-    
+
     // Attributes (+range)
     printf("attribute name                  sup      min      max\n");
     printf("-----------------------------------------------------\n");
     for(int x=0; x<mixer_attributes_count; ++x)
     {
-        VdpBool is_supported = false;
+        VdpBool is_supported;
 
-        rv = device->VideoMixerQueryAttributeSupport(device->device, mixer_attributes[x].id, 
+        rv = device->VideoMixerQueryAttributeSupport(device->device, mixer_attributes[x].id,
             &is_supported);
         is_supported = (rv == VDP_STATUS_OK && is_supported);
-        printf("%-32s %c  ", mixer_attributes[x].name, 
+        printf("%-32s %c  ", mixer_attributes[x].name,
             is_supported?'y':'-');
-        if(is_supported)
+        /* VDPAU spec does not allow range query for DT_NONE types */
+        if(is_supported && mixer_attributes[x].aux != DT_NONE)
         {
             uint32_t minval, maxval;
-            rv = device->VideoMixerQueryAttributeValueRange(device->device, mixer_parameters[x].id, 
+            rv = device->VideoMixerQueryAttributeValueRange(device->device, mixer_parameters[x].id,
                 (void*)&minval, (void*)&maxval);
             if(rv == VDP_STATUS_OK)
                 display_range(mixer_attributes[x].aux, minval, maxval);
         }
         printf("\n");
     }
-    printf("\n");    
+    printf("\n");
 }
 
 /******************* Decoder ****************/
@@ -346,6 +360,23 @@ Desc decoder_profiles[] = {
 {"DIVX5_MOBILE",       VDP_DECODER_PROFILE_DIVX5_MOBILE},
 {"DIVX5_HOME_THEATER", VDP_DECODER_PROFILE_DIVX5_HOME_THEATER},
 {"DIVX5_HD_1080P",     VDP_DECODER_PROFILE_DIVX5_HD_1080P},
+{"H264_CONSTRAINED_BASELINE",   VDP_DECODER_PROFILE_H264_CONSTRAINED_BASELINE},
+{"H264_EXTENDED",               VDP_DECODER_PROFILE_H264_EXTENDED},
+{"H264_PROGRESSIVE_HIGH",       VDP_DECODER_PROFILE_H264_PROGRESSIVE_HIGH},
+{"H264_CONSTRAINED_HIGH",       VDP_DECODER_PROFILE_H264_CONSTRAINED_HIGH},
+{"H264_HIGH_444_PREDICTIVE",    VDP_DECODER_PROFILE_H264_HIGH_444_PREDICTIVE},
+{"VP9_PROFILE_0",               VDP_DECODER_PROFILE_VP9_PROFILE_0},
+{"VP9_PROFILE_1",               VDP_DECODER_PROFILE_VP9_PROFILE_1},
+{"VP9_PROFILE_2",               VDP_DECODER_PROFILE_VP9_PROFILE_2},
+{"VP9_PROFILE_3",               VDP_DECODER_PROFILE_VP9_PROFILE_3},
+{"HEVC_MAIN",                   VDP_DECODER_PROFILE_HEVC_MAIN},
+{"HEVC_MAIN_10",                VDP_DECODER_PROFILE_HEVC_MAIN_10},
+{"HEVC_MAIN_STILL",             VDP_DECODER_PROFILE_HEVC_MAIN_STILL},
+{"HEVC_MAIN_12",                VDP_DECODER_PROFILE_HEVC_MAIN_12},
+{"HEVC_MAIN_444",               VDP_DECODER_PROFILE_HEVC_MAIN_444},
+{"HEVC_MAIN_444_10",            VDP_DECODER_PROFILE_HEVC_MAIN_444_10},
+{"HEVC_MAIN_444_12",            VDP_DECODER_PROFILE_HEVC_MAIN_444_12},
+
 };
 const size_t decoder_profile_count = sizeof(decoder_profiles)/sizeof(Desc);
 
@@ -353,39 +384,118 @@ void queryDecoderCaps(VDPDeviceImpl *device)
 {
     VdpStatus rv;
     printf("\nDecoder capabilities:\n\n");
-    printf("name               level macbs width height\n");
-    printf("-------------------------------------------\n");
+    printf("name                        level macbs width height\n");
+    printf("----------------------------------------------------\n");
     for(int x=0; x<decoder_profile_count; ++x)
     {
-        VdpBool is_supported = false;
+        VdpBool is_supported;
         uint32_t max_level, max_macroblocks, max_width, max_height;
 
-        rv = device->DecoderQueryCapabilities(device->device, decoder_profiles[x].id, 
+        rv = device->DecoderQueryCapabilities(device->device, decoder_profiles[x].id,
             &is_supported, &max_level, &max_macroblocks, &max_width, &max_height);
         if(rv == VDP_STATUS_OK && is_supported)
         {
-            printf("%-20s %2i %5i %5i %5i\n", decoder_profiles[x].name, 
+            printf("%-30s %2i %5i %5i %5i\n", decoder_profiles[x].name,
                 max_level, max_macroblocks, max_width, max_height);
+        } else {
+            printf("%-30s --- not supported ---\n", decoder_profiles[x].name);
         }
     }
 }
 
+static void usage(const char *argv0) _X_NORETURN;
+static void usage(const char *argv0)
+{
+    printf("Usage: %s [--display displayname] [--screen screen_number]\n", argv0);
+    printf("  --display displayname     Server to query\n");
+    printf("  --screen screen_number    X screen to query\n");
+    printf("  --help                    Print this help text\n");
+    exit(0);
+}
+
+static void invalid_argument(const char *argv0, const char *fmt, ...) _X_ATTRIBUTE_PRINTF(2, 3) _X_NORETURN;
+static void invalid_argument(const char *argv0, const char *fmt, ...)
+{
+    va_list ap;
+    va_start(ap, fmt);
+
+    fprintf(stderr, "%s: ", argv0);
+    vfprintf(stderr, fmt, ap);
+    fprintf(stderr, "Try '%s --help' for more information.\n", argv0);
+
+    va_end(ap);
+    exit(-1);
+}
+
+struct Options {
+    char *display_name;
+    int screen;
+};
+
+static struct Options parse_options(int argc, char **argv)
+{
+    struct Options o;
+
+    o.display_name = XDisplayName(NULL);
+    o.screen = -1;
+
+    for(int i = 1; i < argc;)
+    {
+        const char *arg = argv[i++];
+
+        if(strcmp(arg, "--display") == 0)
+        {
+            if(i >= argc)
+                invalid_argument(argv[0], "--display requires an argument\n");
+            o.display_name = XDisplayName(argv[i++]);
+        }
+        else if(strcmp(arg, "--screen") == 0)
+        {
+            if(i >= argc)
+                invalid_argument(argv[0], "--screen requires an argument\n");
+
+            char *end;
+            o.screen = strtol(argv[i], &end, 0);
+            if(end == argv[i] || *end != '\0')
+                invalid_argument(argv[0], "could not parse '%s' as a number\n", argv[i]);
+            if(o.screen < 0)
+                invalid_argument(argv[0], "--screen argument cannot be negative\n");
+            i++;
+        }
+        else if(strcmp(arg, "-h") == 0 || strcmp(arg, "--help") == 0)
+        {
+            usage(argv[0]);
+        }
+        else
+        {
+            invalid_argument(argv[0], "unrecognized option '%s'\n", arg);
+        }
+    }
+
+    return o;
+}
 
 int main(int argc, char **argv)
 {
+    /* Parse options */
+    struct Options o = parse_options(argc, argv);
+
     /* Create an X Display */
-    Display *display;
-    int screen;
-    char *display_name = XDisplayName(NULL);
-    if ((display=XOpenDisplay(display_name)) == NULL)
+    Display *display = XOpenDisplay(o.display_name);
+    if(!display)
     {
-        fprintf(stderr,"vdpauinfo: cannot connect to X server %s\n",
-              XDisplayName(display_name));
+        fprintf(stderr,"%s: cannot connect to X server %s\n", argv[0],
+                o.display_name);
         exit(-1);
     }
-    screen = DefaultScreen(display);
-    printf("display: %s   screen: %i\n", display_name, screen);
-    
+
+    int screen = (o.screen == -1) ? DefaultScreen(display) : o.screen;
+    if(screen >= ScreenCount(display))
+        invalid_argument(argv[0], "screen %d requested but X server only has %d screen%s\n",
+                         screen, ScreenCount(display),
+                         ScreenCount(display) == 1 ? "" : "s");
+    printf("display: %s   screen: %i\n", o.display_name, screen);
+
     /* Create device */
     VdpDevice device;
     VdpGetProcAddress *get_proc_address;
@@ -395,10 +505,10 @@ int main(int argc, char **argv)
     {
         fprintf(stderr, "Error creating VDPAU device: %i\n", rv); /* cannot use GetErrorString here */
         exit(-1);
-    }    
-    
+    }
+
     VDPDeviceImpl *impl = new VDPDeviceImpl(device, get_proc_address);
-    
+
     queryBaseInfo(impl);
     queryVideoSurface(impl);
     queryDecoderCaps(impl);
@@ -408,4 +518,3 @@ int main(int argc, char **argv)
 
     printf("\n");
 }
-
